package
Version:
v0.0.3
Opens a new window with list of versions in this module.
Published: Jul 23, 2025
License: MIT
Opens a new window with license information.
Imports: 0
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
README
¶
union
import "github.com/dashjay/xiter/pkg/union"
Index
type U2
type U2[T1, T2 any] struct {
T1 T1
T2 T2
}
type U3
type U3[T1, T2, T3 any] struct {
T1 T1
T2 T2
T3 T3
}
Generated by gomarkdoc
Documentation
¶
type U2[T1, T2 any] struct {
T1 T1
T2 T2
}
type U3[T1, T2, T3 any] struct {
T1 T1
T2 T2
T3 T3
}
Source Files
¶
Click to show internal directories.
Click to hide internal directories.