package
Version:
v0.0.1
Opens a new window with list of versions in this module.
Published: Jul 13, 2025
License: MIT
Opens a new window with license information.
Imports: 0
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
type U2[T1, T2 any] struct {
T1 T1
T2 T2
}
type U3[T1, T2, T3 any] struct {
T1 T1
T2 T2
T3 T3
}
Source Files
¶
Click to show internal directories.
Click to hide internal directories.